package com.example.ljgscancode

import android.Manifest
import android.content.pm.PackageManager
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.Message
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.camera.core.*
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.example.ljgscancode.databinding.ActivityScanCodeBinding
import com.example.ljgscancode.model.Product
import com.example.ljgscancode.viewmodel.ResultListViewModel
import com.google.mlkit.vision.barcode.BarcodeScanning
import com.google.mlkit.vision.common.InputImage
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import java.lang.ref.WeakReference
import java.nio.ByteBuffer
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors


class ScanCodeActivity : AppCompatActivity() {

    companion object {
        private const val TAG = "CameraXBasic"
        private const val REQUEST_CODE_PERMISSIONS = 10
        private val REQUIRED_PERMISSIONS = arrayOf(Manifest.permission.CAMERA)
    }

    private lateinit var cameraExecutor: ExecutorService
    private lateinit var binding: ActivityScanCodeBinding
    private lateinit var resultList: ResultListViewModel

    private var handler: CodeHandler? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityScanCodeBinding.inflate(layoutInflater)
        setContentView(binding.root)

        /// 请求权限
        if (allPermissionsGranted()) {
            startCamera()
        } else {
            ActivityCompat.requestPermissions(this, REQUIRED_PERMISSIONS, REQUEST_CODE_PERMISSIONS)
        }

        val type = intent.getStringExtra("typeId")!!

        handler = CodeHandler(this)

        binding.resultView.layoutManager =
            GridLayoutManager(this, 3, GridLayoutManager.HORIZONTAL, false)

        resultList = ViewModelProvider(this).get(ResultListViewModel::class.java)

        resultList.getData().observe(this) {
//            if (it.isNotEmpty()) {
//                binding.resultViewBox.visibility = View.VISIBLE
//            }else{
//                binding.resultViewBox.visibility = View.INVISIBLE
//            }
            binding.resultView.adapter = ResultAdapter(it)
        }
        resultList.initData(this, typeId = type)

        binding.intoStock.setOnClickListener {
            resultList.intoStock(this)
        }

        cameraExecutor = Executors.newSingleThreadExecutor()
    }


    private fun startCamera() {
        val cameraProviderFuture = ProcessCameraProvider.getInstance(this)

        cameraProviderFuture.addListener({
            val cameraProvider: ProcessCameraProvider = cameraProviderFuture.get()
            val cameraSelector = CameraSelector.DEFAULT_BACK_CAMERA
            val preview = Preview.Builder().build().also {
                it.setSurfaceProvider(binding.viewFinder.surfaceProvider)
            }

            val mlScancode = ImageAnalysis.Builder().build().also {
                it.setAnalyzer(cameraExecutor, MLScanCodeAnalyzer(handler))
            }

            try {
                cameraProvider.unbindAll()

                cameraProvider.bindToLifecycle(
                    this, cameraSelector, preview, mlScancode
                )

            } catch (exc: Exception) {
                Log.e(TAG, "Use case binding failed", exc)
            }

        }, ContextCompat.getMainExecutor(this))
    }

    fun addResult(result: String) {
        resultList.addResult(result)
    }

    private fun allPermissionsGranted(): Boolean {
        return REQUIRED_PERMISSIONS.all {
            ContextCompat.checkSelfPermission(
                baseContext, it
            ) == PackageManager.PERMISSION_GRANTED
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray,
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == REQUEST_CODE_PERMISSIONS) {
            if (allPermissionsGranted()) {
                startCamera()
            } else {
                Toast.makeText(
                    this,
                    "请去设置里授予程序相机权限",
                    Toast.LENGTH_SHORT
                ).show()
            }
        }
    }


    override fun onDestroy() {
        super.onDestroy()
        cameraExecutor.shutdown()
        handler = null
    }


    private class MLScanCodeAnalyzer(val handler: CodeHandler?) : ImageAnalysis.Analyzer {
        private var isAnalyze: Boolean = false;

        private fun ImageProxy.toInputImage(byteBuffer: ByteBuffer): InputImage {

            return InputImage.fromByteBuffer(
                byteBuffer,
                this.width,
                this.height,
                this.imageInfo.rotationDegrees,
                InputImage.IMAGE_FORMAT_YV12
            )
        }


        override fun analyze(image: ImageProxy) {
            val buffer = image.planes[0].buffer

            try {
                scanBarcodes(image.toInputImage(buffer))
            } catch (e: Exception) {
                Log.d(TAG, "analyze with exception:${e.stackTrace}")
            } finally {
                buffer.clear()
                image.close()
            }
        }

        private fun scanBarcodes(image: InputImage) {
            if (isAnalyze) return;
            isAnalyze = true;
            val scanner = BarcodeScanning.getClient()
            val result = scanner.process(image)
                .addOnSuccessListener { barcodes ->
                    for (barcode in barcodes) {
                        if (handler != null) {
                            val message = handler.obtainMessage(1, barcode.rawValue)
                            handler.sendMessage(message)
                        }
//                        Log.d(TAG, "code scan result:  ${barcode.rawValue}")
                    }
                }
                .addOnFailureListener {
                    // Task failed with an exception
                    // ...
                }
            isAnalyze = false;
        }
    }

    class CodeHandler(activity: ScanCodeActivity) : Handler(Looper.getMainLooper()) {
        private val activity: WeakReference<ScanCodeActivity> = WeakReference(activity)

        override fun handleMessage(msg: Message) {
            super.handleMessage(msg)

            if (msg.what == 1) {
                activity.get()?.addResult(msg.obj as String)
            } else {

            }
        }
    }

    private class ResultAdapter(val results: List<String>) :
        RecyclerView.Adapter<ResultAdapter.ResultViewHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ResultViewHolder {
            val view =
                LayoutInflater.from(parent.context).inflate(R.layout.item_result, parent, false)
            return ResultViewHolder(view)
        }

        override fun onBindViewHolder(holder: ResultViewHolder, position: Int) {
            val result = results[position]

            holder.textView.text = result
        }

        override fun getItemCount(): Int = results.size


        class ResultViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
            var textView: TextView = itemView.findViewById(R.id.resultText)
        }
    }
}